﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Stormtrooper
{
    internal abstract class AbstractMap
    {
        private IDrawningObject _drawningObject = null;
        protected int[,] _map = null;
        protected int _width;
        protected int _height;
        protected float _size_x;
        protected float _size_y;
        protected readonly Random _random = new();
        protected readonly int _freeRoad = 0;
        protected readonly int _barrier = 1;
        public Bitmap CreateMap(int width, int height, IDrawningObject
       drawningObject)
        {
            _width = width;
            _height = height;
            _drawningObject = drawningObject;
            GenerateMap();
            while (!SetObjectOnMap())
            {
                GenerateMap();
            }
            return DrawMapWithObject();
        }
        public Bitmap MoveObject(Direction direction)
        {
            int LefTopX = Convert.ToInt32(_drawningObject.GetCurrentPosition().Left / _size_x);
            int LefTopY = Convert.ToInt32(_drawningObject.GetCurrentPosition().Top / _size_y);
            int objwidth = Convert.ToInt32(_drawningObject.GetCurrentPosition().Right / _size_x);
            int objheigh = Convert.ToInt32(_drawningObject.GetCurrentPosition().Bottom / _size_y);

            bool CanStep = true;
            switch (direction)
            {
                case Direction.Left:
                    for (int i = LefTopX; i >= Math.Abs(LefTopX - Convert.ToInt32(_drawningObject.Step / _size_x)); i--)
                    {
                        for (int j = LefTopY; j <= objheigh && j<_map.GetLength(1); j++)
                        {
                            
                            if (_map[i, j] == _barrier)
                            {
                                CanStep = false;
                                break;
                            }
                        }
                    }
                    break;

                case Direction.Right:
                    for (int i = objwidth; i <= objwidth + Convert.ToInt32(_drawningObject.Step / _size_x) && i < _map.GetLength(0); i++)
                    {
                        for (int j = LefTopY; j <= objheigh && j < _map.GetLength(1); j++)
                        {
                            if (_map[i, j] == _barrier)
                            {
                                CanStep = false;
                                break;
                            }
                        }
                    }
                    
                    break;

                case Direction.Down:
                    for (int i = LefTopX; i <= objwidth && i < _map.GetLength(0); i++)
                    {
                        for (int j = objheigh; j <= objheigh + Convert.ToInt32(_drawningObject.Step / _size_y) && j < _map.GetLength(1); j++)
                        {

                            if (_map[i, j] == _barrier)
                            {
                                CanStep = false;
                                break;
                            }
                        }
                    }
                    
                    break;
                case Direction.Up:
                    for (int i = LefTopX; i <= objwidth && i<_map.GetLength(0); i++)
                    {
                        for (int j = LefTopY; j >= Math.Abs(LefTopY - Convert.ToInt32(_drawningObject.Step / _size_y)) ; j--)
                        {
                            if (_map[i, j] == _barrier)
                            {
                                CanStep = false;
                                break;
                            }
                        }
                    }
                    break;
            }

            if (CanStep)
            {
                _drawningObject.MoveObject(direction);
            }
            return DrawMapWithObject();
        }
        private bool SetObjectOnMap()
        {
            if (_drawningObject == null || _map == null)
            {
                return false;
            }
            int x = _random.Next(0, 150);
            int y = _random.Next(100, 300);

            for (int i = 0; i < _map.GetLength(0); ++i)
            {
                for (int j = 0; j < _map.GetLength(1); ++j)
                {
                    if (i * _size_x >= x && j * _size_y <= y &&
                        i * _size_x <= x + _drawningObject.GetCurrentPosition().Right &&
                        j * _size_y >= j - _drawningObject.GetCurrentPosition().Bottom && _map[i, j] == _barrier)
                    {
                        return false;
                    }
                }
            }

            _drawningObject.SetObject(x, y, _width, _height);

            return true;
        }
        private Bitmap DrawMapWithObject()
        {
            Bitmap bmp = new(_width, _height);
            if (_drawningObject == null || _map == null)
            {
                return bmp;
            }
            Graphics gr = Graphics.FromImage(bmp);
            for (int i = 0; i < _map.GetLength(0); ++i)
                for (int j = 0; j < _map.GetLength(1); ++j)
                     DrawRoadPart(gr, i, j);

            for (int i = 0; i < _map.GetLength(0); ++i)
                for (int j = 0; j < _map.GetLength(1); ++j)
                    if (_map[i, j] == 1) DrawBarrierPart(gr, i, j);

            _drawningObject.DrawningObject(gr);
            return bmp;
        }
        protected abstract void GenerateMap();
        protected abstract void DrawRoadPart(Graphics g, int i, int j);
        protected abstract void DrawBarrierPart(Graphics g, int i, int j);
    }
}
