﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Security.Policy;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Stormtrooper
{
    /// <summary>
    /// Класс, отвечающий за прорисовку и перемещение объекта-сущности
    /// </summary>
    public class Drawning
    {
        public EntityStormtrooper Storm { get; protected set; }
        /// <summary>
        /// Левая координата отрисовки самолёта
        /// </summary>
        protected float _startPosX;
        /// <summary>
        /// Нижняя кооридната отрисовки самолёта
        /// </summary>
        protected float _startPosY;
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private int? _pictureWidth = null;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private int? _pictureHeight = null;
        /// <summary>
        /// Ширина отрисовки самолёта
        /// </summary>
        private readonly int _StormWidth = 135;
        /// <summary>
        /// Высота отрисовки самолёта
        /// </summary>
        private readonly int _StormHeight = 100;
        /// <summary>
        /// Инициализация свойств
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес самолёта</param>
        /// <param name="bodyColor">Цвет кузова</param>
        public Drawning(int speed, float weight, Color bodyColor) 
        {
            Storm = new EntityStormtrooper(speed, weight, bodyColor);
        }
        /// <summary>
        /// Инициализация свойств
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес самолёта</param>
        /// <param name="bodyColor">Цвет кузова</param>
        /// <param name="carWidth">Ширина отрисовки самолёта</param>
        /// <param name="carHeight">Высота отрисовки самолёта</param>
        protected Drawning(int speed, float weight, Color bodyColor, int stormWidth, int stormHeight) :
        this(speed, weight, bodyColor)
        {
            _StormWidth = stormWidth;
            _StormHeight = stormHeight;
        }
        /// <summary>
        /// Установка позиции самолёта
        /// </summary>
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public void SetPosition(int x, int y, int width, int height)
        {

            if (x < 0 || y - _StormHeight < 0 || x + _StormWidth > width || y > height)
            {
                return;
            }
            
            _startPosX = x;
           _startPosY = y;
            
            _pictureWidth = width;
            _pictureHeight = height;
        }
        /// <summary>
        /// Изменение направления перемещения
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveTransport(Direction direction)
        {
            if (!_pictureWidth.HasValue || !_pictureHeight.HasValue)
            {
                return;
            }
            switch (direction)
            {
                // вправо
                case Direction.Right:
                    if (_startPosX + _StormWidth + Storm.Step < _pictureWidth)
                    {
                        _startPosX += Storm.Step;
                    }
                    break;
                //влево
                case Direction.Left:
                    if (_startPosX - Storm.Step > 0)
                    {
                        _startPosX -= Storm.Step;
                    }
                    break;
                //вверх
                case Direction.Up:
                    if (_startPosY - _StormHeight - Storm.Step > 0)
                    {
                        _startPosY -= Storm.Step;
                    }
                    break;
                //вниз
                case Direction.Down:
                    if (_startPosY + Storm.Step < _pictureHeight)
                    {
                        _startPosY += Storm.Step;
                    }
                    break;
            }
        }
        /// <summary>
        /// Отрисовка штурмовика
        /// </summary>
        /// <param name="g"></param>
        public virtual void DrawTransport(Graphics g)
        {
            if (_startPosX < 0 || _startPosY < 0
                || !_pictureHeight.HasValue || !_pictureWidth.HasValue)
            {
                return;
            }

            Pen pen = new(Color.Black);
            //крыло
            Point[] F =
            {
              new Point((int)_startPosX + 60 , (int)_startPosY),
              new Point((int)_startPosX + 80 , (int)_startPosY),
              new Point((int)_startPosX+90 , (int)_startPosY - 40),
              new Point((int)_startPosX+90 , (int)_startPosY - 60),
              new Point((int)_startPosX+80 , (int)_startPosY - 100),
              new Point((int)_startPosX+60 , (int)_startPosY - 100),
            };
            g.DrawPolygon(pen, F);
            //фюзеляж самолёта
            g.DrawRectangle(pen, _startPosX + 20, _startPosY - 60, 100, 20);
            //нос самолёта
            Point[] N =
            {
              new Point((int)_startPosX + 20 , (int)_startPosY - 40),
              new Point((int)_startPosX + 20 , (int)_startPosY - 60),
              new Point((int)_startPosX , (int)_startPosY - 50),
            };
            g.DrawPolygon(pen, N);
            //стабилизатор
            Point[] S =
            {
              new Point((int)_startPosX + 120 , (int)_startPosY-40),
              new Point((int)_startPosX + 135 , (int)_startPosY-20),
              new Point((int)_startPosX+135 , (int)_startPosY - 80),
              new Point((int)_startPosX+120 , (int)_startPosY - 60),
            };
            g.DrawPolygon(pen, S);

            //нос
            Brush brBl = new SolidBrush(Color.Black);
            g.FillPolygon(brBl, N);

            //фюзеляж
            Brush brGr = new SolidBrush(Color.White);
            g.FillRectangle(brGr, _startPosX + 20, _startPosY - 60, 100, 20);

            //крыло и стаб
            Brush br = new SolidBrush(Storm?.BodyColor ?? Color.Black);
            g.FillPolygon(br, F);
            g.FillPolygon(br, S);

        }

        /// <summary>
        /// Смена границ формы отрисовки
        /// </summary>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public void ChangeBorders(int width, int height)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            if (_pictureWidth <= _StormWidth || _pictureHeight <= _StormHeight)
            {
                _pictureWidth = null;
                _pictureHeight = null;
                return;
            }
            if (_startPosX + _StormWidth > _pictureWidth)
            {
                _startPosX = _pictureWidth.Value - _StormWidth;
            }
            if (_startPosY + _StormHeight > _pictureHeight)
            {
                _startPosY = _pictureHeight.Value - _StormHeight;
            }
        }
        /// <summary>
        /// Получение текущей позиции объекта
        /// </summary>
        /// <returns></returns>
        public (float Left, float Right, float Top, float Bottom) GetCurrentPosition()
        {
            return (_startPosX, _startPosX + _StormWidth, _startPosY - _StormHeight, _startPosY);
        }
    }
}
