﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Stormtrooper
{
    /// <summary>
    /// Форма создания объекта
    /// </summary>
    public partial class FormStormtrooperConfig : Form
    {
        /// <summary>
        /// Переменная-выбранный самолёт
        /// </summary>
        Drawning _storm = null;
        /// <summary>
        /// Делегат
        /// </summary>
        public delegate void Action (Drawning storm);
        /// <summary>
        /// Событие
        /// </summary>
        private event Action EventAddStorm;
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormStormtrooperConfig()
        {
            InitializeComponent();
            panelBlack.MouseDown += PanelColor_MouseDown;
            panelPurple.MouseDown += PanelColor_MouseDown;
            panelGray.MouseDown += PanelColor_MouseDown;
            panelGreen.MouseDown += PanelColor_MouseDown;
            panelRed.MouseDown += PanelColor_MouseDown;
            panelWhite.MouseDown += PanelColor_MouseDown;
            panelYellow.MouseDown += PanelColor_MouseDown;
            panelBlue.MouseDown += PanelColor_MouseDown;

            buttonCancel.Click += (object sender, EventArgs a) => Close();
        }
        /// <summary>
        /// Отрисовать машину
        /// </summary>
        private void DrawStorm()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _storm?.SetPosition(5, 105, pictureBoxObject.Width, pictureBoxObject.Height);
            _storm?.DrawTransport(gr);
            pictureBoxObject.Image = bmp;
        }

        /// <summary>
        /// Добавление события
        /// </summary>
        /// <param name="ev"></param>
        public void AddEvent(Action ev)
        {
            if (EventAddStorm == null)
            {
                EventAddStorm = new Action(ev);
            }
            else
            {
                EventAddStorm += ev;
            }
        }
        /// <summary>
        /// Передаем информацию при нажатии на Label
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label).DoDragDrop((sender as Label).Name, DragDropEffects.Move | DragDropEffects.Copy);
        }
        /// <summary>
        /// Проверка получаемой информации (ее типа на соответствие требуемому)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.Text))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        /// <summary>
        /// Действия при приеме перетаскиваемой информации
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data.GetData(DataFormats.Text).ToString())
            {
                case "labelSimpleObject":
                    _storm = new Drawning((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, Color.White);
                    break;
                case "labelModifiedObject":
                    _storm = new DrawningMilitary((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, Color.White, Color.Black,
                        checkBoxBodyKit.Checked, checkBoxRocket.Checked, checkBoxSportLine.Checked);
                    break;
            }
            DrawStorm();
        }
        /// <summary>
        /// Отправляем цвет с панели
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelColor_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Control).DoDragDrop((sender as Control).BackColor, DragDropEffects.Move | DragDropEffects.Copy);
        }
        /// <summary>
        /// Проверка получаемой информации (ее типа на соответствие требуемому)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelBaseColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        /// <summary>
        /// Принимаем основной цвет
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelBaseColor_DragDrop(object sender, DragEventArgs e)
        {
            var color = e.Data.GetData(typeof(Color));
            if (color != null)
            {
                (sender as Label).BackColor = (Color)color;
            }
            if (_storm != null)
            {
                _storm.Storm.BodyColor = (Color)color;
                DrawStorm();
            }
        }
        /// <summary>
        /// Принимаем дополнительный цвет
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelDopColor_DragDrop(object sender, DragEventArgs e)
        {
            var color = e.Data.GetData(typeof(Color));
            if (color != null)
            {
                (sender as Label).BackColor = (Color)color;
            }
            if (_storm != null)
            {
                if (_storm is DrawningMilitary storm)
                {
                    storm.SetDopColor((Color)color);
                    DrawStorm();
                }
                DrawStorm();
            }
        }
        /// <summary>
        /// Добавление самолёта
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonOk_Click(object sender, EventArgs e)
        {
            EventAddStorm?.Invoke(_storm);
            Close();
        }
    }
}
