﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Stormtrooper
{
    /// <summary>
    /// Параметризованный набор объектов
    /// </summary>
    /// <typeparam name="T"></typeparam>
    internal class SetStormtroopersGeneric<T>
        where T : class
    {
        /// <summary>
        ///  Список объектов, которые храним
        /// </summary>
        private readonly List<T> _places;
        /// <summary>
        /// Количество объектов в списке
        /// </summary>
        public int Count => _places.Count;

        private readonly int _maxCount;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetStormtroopersGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T>();
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="stormtrooper">Добавляемый штурмовик</param>
        /// <returns></returns>
        public int Insert(T stormtrooper)
        {
            if (_places.Count + 1 >= _maxCount) return -1;
            return Insert(stormtrooper, 0);
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="stormtrooper">Добавляемый штурмовик</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public int Insert(T stormtrooper, int position)
        {
            if (position < 0 || position >= _maxCount) return -1;
            if (_places.Count + 1 >= _maxCount) return -1;
            _places.Insert(position, stormtrooper);
            return position;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T Remove(int position)
        {
            if (position < 0 || position >= _maxCount) return null;
            T saveStorm = _places[position];
            _places.RemoveAt(position);
            return saveStorm;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T this[int position]
        {
            get
            {
                if (position < 0 || position >= _maxCount) return null;
                return _places[position];
            }
            set
            {
                if (position < 0 || position >= _maxCount) Insert(value, position);

            }
        }
        /// <summary>
        /// Проход по набору до первого пустого
        /// </summary>
        /// <returns></returns>
        public IEnumerable<T> GetStormtroopers()
        {
            foreach (var storm in _places)
            {
                if (storm != null)
                {
                    yield return storm;
                }
                else
                {
                    yield break;
                }
            }
        }
    }
}
